list_ = [
    r"gold(?:en)?",
    "dark",
    "hearty",
    "rich",
    "homemade",
    "tasty",
    "delicious",
    "yummy",
    "good",
    "cheap",
    "expensive",
    "entire",
    "leftover",
    "quick",
    "easy",
    "simple",
    "fine",
    r"\w+[- ]?sodium",
    "creamy",
    "organic",
    "broken",
    "all-butter",
    "sweet",
    "mild",
    "sharp",
    "firm",
    "soft",
    "aged",
    "thin",
    "thick",
    "lean",
    "stoneground",
    r"\w+[- ]?style",
    r"(?:[0-9]+ [0-9]+/[0-9]+|[0-9]+)[- ]?inch(?:[ -]thick)?",
    r"\w+-sodium",
    r"\w+-fat",
    r"free[- ]range",
    "whole",
    "skin-on",
    "hot",
    "cold",
    "warm",
    "boiling",
    r"\b(?<!\.)(?!0+(?:\.0+)?[ -]?%)(?:\d|[1-9]\d|100)(?:(?<!100)\.\d+)?[ -]?%",
    r"fresh(?:ly)?",
    r"[0-9]+[- ]bone",
    "quality",
    "chopped",
    "ground",
    "sliced",
    "minced",
    "cut",
    "diced",
    "peeled",
    "grated",
    "shredded",
    "drained",
    "dry",
    "coarse",
    "crushed",
    "crusted",
    "dried",
    "cooked",
    "boneless",
    "seeded",
    "beaten",
    "skinless",
    "unsalted",
    "cubed",
    "rinsed",
    "melted",
    "softened",
    "canned",
    "toasted",
    "crumbled",
    "quartered",
    "uncooked",
    "roasted",
    "trimmed",
    "pitted",
    "thawed",
    "granulated",
    "frozen",
    "washed",
    "ripe",
    "mixed",
    "powdered",
    "cored",
    "unsweetened",
    "condensed",
    "smoked",
    "deveined",
    "reduced",
    "mashed",
    "halved",
    "separated",
    "boiled",
    "raw",
    "undrained",
    "seasoned",
    "sifted",
    "slivered",
    "juiced",
    "squeezed",
    "sweetened",
    "evaporated",
    "whipped",
    "blanched",
    "chilled",
    "julienned",
    "cracked",
    "prepared",
    "shelled",
    "stemmed",
    "refried",
    "stewed",
    "unbleached",
    "flaked",
    "skinned",
    "flavored",
    "refrigerated",
    "semisweet",
    "unpeeled",
    "smashed",
    "salted",
    "pickled",
    "packed",
    "snipped",
    "seedless",
    "bottled",
    "deseeded",
    "soaked",
    "rolled",
    "cooled",
    "candied",
    "warmed",
    "baked",
    "scrubbed",
    "steamed",
    "distilled",
    "pressed",
    "corned",
    "marinated",
    "brewed",
    "undiluted",
    "shaved",
    "clarified",
    "stuffed",
    "pureed",
    "unflavored",
    "desiccated",
    "creamed",
    "grilled",
    "zested",
    "hulled",
    "substituted",
    "defrosted",
    "unbaked",
    "assorted",
    "heated",
    "husked",
    "preserved",
    "strained",
    "browned",
    "scraped",
    "thickened",
    "processed",
    "cleaned",
    "jarred",
    "bruised",
    "unseasoned",
    "chunked",
    "crystallized",
    "scalded",
    "rubbed",
    "precooked",
    "buttered",
    "carbonated",
    "stoned",
    "sauteed",
    "tied",
    "shucked",
    "unsifted",
    "butterflied",
    "bone-in",
    "colored",
    "debearded",
    "cured",
    "spiced",
    "spicy",
    "converted",
    "imported",
    "seperated",
    "tinned",
    "blended",
    "added",
    "whisked",
    "deboned",
    "hardboiled",
    "shaped",
    "purchased",
    "fermented",
    "sectioned",
    "veined",
    "suggested",
    "unsliced",
    "pounded",
    "ripened",
    "parboiled",
    "packaged",
    "sorted",
    "textured",
    "dehydrated",
    "grounded",
    "refined",
    "filtered",
    "poached",
    "chipped",
    "compressed",
    "segmented",
    "sieved",
    "heaped",
    "jointed",
    "flattened",
    "combined",
    "unrefined",
    "gutted",
    "caramelized",
    "sprinkled",
    "scaled",
    "pulled",
    "desired",
    "based",
    "rendered",
    "tailed",
    "unblanched",
    "brushed",
    "stripped",
    "diluted",
    "stirred",
    "homogenized",
    "barbecued",
    "concentrated",
    "wedged",
    "herbed",
    "rehydrated",
    "breaded",
    "deviened",
    "scored",
    "rounded",
    "wiped",
    "posted",
    "defatted",
    "filleted",
    "frenched",
    "skin-on",
    "unwaxed",
    "reconstituted",
    "tossed",
    "flavoured",
    "unshelled",
    "choped",
    "tenderized",
    "charred",
    "pasteurized",
    "malted",
    "unsmoked",
    "filled",
]


def flatten(t):
    return [item for sublist in t for item in sublist]


portions = {
    "chicken": [
        r"wings?",
        r"giblets?",
        r"necks?",
        r"thighs?",
        r"legs?",
        r"drumsticks?",
        r"breasts?",
        r"back",
        r"drumm?ettes?",
        r"fillets?",
        r"feet",
        r"hindquarters?",
        r"rumps?",
        r"gizzards?",
        r"tenders?",
        r"nuggets?",
        r"quarters?",
        r"hal(?:f|ves)",
        r"meat",
        r"pieces?",
        r"cutlet",
        r"parts?",
        r"strips?",
        r"portions?",
        r"skin",
        r"carcass",
        r"chunks?",
        r"mince",
        r"hearts?",
        r"escalope",
    ],
    "fish": [
        r"fillets?",
        r"steaks?",
        r"fins?",
        r"fish",
        r"cake",
        r"pie",
        r"tail",
        # these are cheap, think bout it
        r"trimmings?",
        r"sticks?",
        r"fingers?",
        r"meat",
        r"cheeks?",
        r"loin",
        r"head",
        r"balls?",
        r"carcass",
        r"medallions?",
        r"scampi",
    ],
    "pork": [
        r"trotters?",
        r"shank",
        r"legs?",
        r"hogs?",
        r"loins?",
        r"bell(?:y|ies)",
        r"fillets?",
        r"shoulders?",
        r"necks?",
        r"heads?",
        r"spare[- ]?ribs?",
        r"ribs?",
        r"meat",
        r"butt",
        r"chop",
        r"mince",
        r"hock",
        r"rind",
        r"cheek",
        r"skin",
        r"escalope",
        r"medallions?",
        r"rack",
        r"fatback",
        r"kidney",
        r"schnitzel",
        r"breast",
        r"brisket",
        r"cutlets?",
    ],
    "beef": [
        r"shin",
        r"silver[- ]?side",
        r"kidneys?",
        r"top[- ]?side",
        r"short[- ]?rib",
        r"prime[- ]?rib",
        r"t[- ]?bone",
        r"fillet",
        r"rump",
        r"flank",
        r"steak",
        r"hump",
        r"neck",
        r"bolo",
        r"wing[- ]?rib",
        r"chuck",
        r"brisket",
        r"round",
        r"meat",
        r"mince",
        r"oxtail",
        r"shoulder",
        r"heart",
        r"sirloin",
        r"jerky",
        r"tongue",
        r"ribs?",
        r"chunks?",
        r"shank",
        r"medallions?",
        r"rump",
    ],
    "lamb": [
        r"neck",
        r"shoulder",
        r"chop",
        r"rack",
        r"loin",
        r"leg",
        r"shank",
        r"meat",
        r"mince",
        r"kidney",
        r"rump",
        r"fillet",
        r"cutlet",
        r"belly",
        r"rib",
        r"breast",
    ],
    "sausage": [r"links?", r"rings?", r"casings?", r"meat", r"patty"],
    "general": [
        "mince",
        "meat",
        "chunk",
        "slice",
        "portion",
        "escalope",
        "medallion",
        "paste",
        "mincemeat",
        "shin",
        "cutlet",
    ],
}
portions["all"] = flatten(list(portions.values()))
portions["any"] = [""]
portions["none"] = [r"$"]

dictionary = {
    "liver": [r"\blivers?\b"],
    "sausage": [
        r"sausage(?:s|meat)?",
        r"chorizo",
        r"kielbasas?",
        r"bratwursts?",
        r"longaniza",
        r"sai[- ]?ua",
        r"longganisa",
        r"lau[- ]?lau",
        r"chipolatas?",
    ],
    "poultry": [r"chicken", "turkey", "rabbit", "hen"],
    "lamb": ["lamb", "mutton", "goat"],
    "game bird": ["pigeon", "pheasant", "quail", "duck", "goose"],
    "game": ["venison", "deer", r"boar", r"game", r"bear", r"kangaroo"],
    "pork": ["pork", "veal", r"meatballs?"],
    "tuna": [r"tuna"],
    "salmon": [r"\bsalmon\b"],
    "fatty fish": [
        "eel",
        "trout",
        "arctic char",
        "butterfish",
        "mackerel",
        r"^anchov(?:y|ies)$",
        "sardine",
        "swordfish",
        "shark",
        r"monk[- ]?fish",
    ],
    "lean fish": [
        "herring",
        "carp",
        "bluefish",
        "bonita",
        "rock bass",
        "squeteague",
        "catfish",
        "flounder",
        "cod",
        "sole",
        "red snapper",
        "snapper",
        "bass",
        "sea bass",
        "perch",
        "haddock",
        "halibut",
        "pike",
        "tilapia",
        "swai",
        "whitefish",
        "fish",
    ],
    "octopus": ["octopus", "squid", "calamari", r"cuttle[- ]?fish"],
    "crustacean": [
        "lobster",
        r"cra[wy][- ]?fish",
        "prawn",
        "shrimp",
        r"cra[wy][- ]?daddy",
        "krill",
        "crab",
        r"shell[- ]?fish",
    ],
    "other seafood": [
        r"clams?",
        r"mussels?",
        r"scallops?",
        r"oysters?",
        r"cockles?",
        r"snails?",
        r"escargots?",
        r"caviar",
        r"\broe\b",
    ],
    "ham": [
        r"mortadella",
        r"\bham$",
        r"pancetta(?: (?:di )?cubetti)?",
        "prosciutto",
        r"jamon(?: iberico)?",
        "jambon",
        "capicola",
        "culatello",
        "gammon",
        "serrano",
    ],
    "beef": [
        "new york strip",
        "cow",
        r"fill?et mignon",
        "loin",
        "tenderloin",
        "sirloin",
        r"beef[^s]",
        "wagyu",
        r"rib[ \-]eye",
        r"beef$",
        "steak",
        "ground meat",
        "mincemeat",
        "stew meat",
        "minced meat",
        "meat",
        r"roast$",
        r"^chuck$",
        r"hamburger$",
    ],
    "bacon": [
        r"bacon bits?",
        r"bacon$",
        r"lardons?",
        r"rashers?",
        r"speck",
        r"guanciale",
        r"szalonna",
        r"lap yuk",
    ],
}

type_portions = {
    "poultry": "chicken",
    "lamb": "lamb",
    "game bird": "chicken",
    "game": "beef",
    "pork": "pork",
    "tuna": "fish",
    "salmon": "fish",
    "fatty fish": "fish",
    "lean fish": "fish",
    "beef": "beef",
    "crustacean": "fish",
    "sausage": "sausage",
    "bacon": "any",
    "ham": "any",
}

exceptions = [
    "stock",
    "juice",
    "bouillon",
]



# from utils import with_plural, to_regex
from ..utils import with_plural, to_regex

# from data.ingredient_categories import fruits, vegetables, legumes, herbs
from .ingredient_categories import fruits, vegetables, legumes, herbs

# from data.meat import dictionary as meat_dict
from .meat import dictionary as meat_dict

# from data.ingredients import dictionary as ing_dict
from .ingredients import dictionary as ing_dict
import re

proteins = list(meat_dict.keys())
pasta = ing_dict["pasta"]

drink_list = with_plural(
    [
        "slushy",
        "slushie",
        "colada",
        "old fashioned",
        "bitter",
        "whiskey",
        "liqueur",
        "cider",
        "cappuccino",
        "frappuccino",
        "coffee",
        "espresso",
        "cocktail",
        "beer",
        "wine",
        "rootbeer",
        "drink",
        "juice",
        "tea",
        "milk",
        "moccha",
        "shake",
        "milkshake",
        "smoothie",
        "cocoa",
        "margarita",
        "mojito",
        "smoothie",
        "gin",
        "lemonade",
        "martini",
        "soda",
        "punch",
        "fizzy",
        "ouzo",
        "mocha",
        "latte",
        "frappe",
        "limeade",
        "cafe",
        "irish cream",
        "café",
        "frappé",
        "cordial",
        "chai",
        "lassi",
        "aqua thunder",
        "limoncello",
        "tequini",
        "cactus bite",
        "the yellow jacket",
        "caribbean twist",
        "brazillian sunset",
    ]
)
soup_list = with_plural(
    [
        "minestrone",
        "soup",
        "chowder",
        "bouillabaisse",
        "gazpacho",
        "bisque",
        "bouillon",
        "broth",
        "consomme",
        "consommé",
        "velouté",
        "veloute",
        "vichyssoise",
        "skink",
        "laksa",
        "ramen",
        "goulash",
        "stew",
        "bourguignon",
        "rogan josh",
    ]
)
niche_list = with_plural(
    [
        "vinaigrette",
        "horseradish",
        "compote",
        "sauce",
        "marinade",
        "dip",
        "jam",
        "queso",
        "jus",
        "dressing",
        "bread",
        "spread",
        "preserve",
        "stock",
        "bouillon",
        "salsa",
        "gravy",
    ]
)
dessert_list = with_plural(
    [
        "apple pie",
        "semifreddo",
        "waffle",
        "crepe",
        "pudding",
        "french toast",
        "torta",
        "tiramisu",
        "sundae",
        "madeleine",
        "ice cream",
        "cookie",
        r"muffin",
        r"bar",
        r"cupcake",
        r"brownie",
        r"cheesecake",
        r"pancake",
        "tart",
        "rugelach",
        "chiffon cake",
        r"biscuit",
        r"scone",
        "sorbet",
        "biscotti",
        "galette",
        "granola",
        "buttercream",
        r"doughnut",
        "crumble",
        "shortbread",
        r"mousse",
        "baklava",
        "donut",
    ]
)
sandwich_list = with_plural(
    [
        "panini",
        "sandwich",
        "burrito",
        "burger",
        "taco",
        "bruschetta",
        "baguette",
        "wrap",
        r"hot ?dog",
        "enchilada",
        "ciabatta",
    ]
)
exceptions = ["stock", "broth", "bouillon", "beefsteak"]

title_map = {
    "seafood": to_regex(["ceviche"]),
    "sandwich": to_regex(sandwich_list),
    "pasta": to_regex(pasta + ["pasta", "alfredo"]),
    "sausage": to_regex(["sausage", "sausages", "chorizo"]),
    "casserole": to_regex(["casserole"]),
    # "dessert": to_regex(dessert_list),
    "soup": to_regex(soup_list),
    "pizza": to_regex([r"pizza$", r"calzone$", r"buffalina$", r"margherita$"]),
}

ing_keys = proteins + ["pasta"]

ing_map = {"spicy food": "spicy pepper"}

root_map = {
    "sandwich": to_regex(sandwich_list),
    "vegetarian": to_regex(
        with_plural(
            vegetables + ["mushroom", "portobell", "brussels sprouts", "tofu", "beans"]
        )
    ),
    "salad": to_regex(["(?<!fruit )salad"]),
    "pasta": to_regex(pasta + ["pasta", "alfredo"]),
    # "dessert": to_regex(dessert_list),
    "niche": to_regex(niche_list),
    "soup": to_regex(soup_list),
    "drink": rf"{to_regex(drink_list)}$",
    "pizza": to_regex(["pizza", "calzone", "buffalina", "margherita"]),
    "sushi": to_regex(
        ["nigiri", "sashimi", "chirashi", "oshizushi", "temaki", "uramaki", "sushi"]
    ),
}


def is_dessert(parsedPhrases):
    def is_dessert_ing(name, quantity):
        key_list = [
            "syrup",
            "molasses",
            "chocolate",
            "caramel",
            r"nut$",
            "cocoa",
            # "sugar",
            "milk",
            "cinnamon",
            "vanilla",
            "honey",
        ]
        if name == "sugar" and quantity and quantity > 50:
            return True

        if re.search(rf"\b(?:{r'|'.join(key_list)})\b", name):
            return True
        return False

    def is_non_dessert_ing(name):
        # herb_exceptions = r"(?:mint|basil|rosemary|thyme)"

        key_list = (
            # [h for h in herbs if not re.search(herb_exceptions, h)]
            proteins
            + ["pepper", "garlic", "onion", "shallot", "spinach", "potato", "tofu"]
            + [
                "monterey jack cheese",
                "mexican cheese",
                "feta cheese",
                "mozzarella cheese",
                "pecorino cheese",
                "gruyere cheese",
                "cottage cheese",
                "pepper jack cheese",
                "cheddar cheese",
                "parmesan cheese",
            ]
        )
        if re.search(rf"\b(?:{r'|'.join(key_list)})\b", name):
            return True
        return False

    if all(
        not ing or not is_dessert_ing(ing["simple"], ing["quantity"])
        for ing in parsedPhrases
    ):
        return False

    if any(ing and is_non_dessert_ing(ing["simple"]) for ing in parsedPhrases):
        return False

    return True


function_map = {"dessert": is_dessert}


models = {
    "fish": [
        "seafood",
        "tuna",
        "lean fish",
        "salmon",
        "crustacean",
        "fatty fish",
        "other seafood",
        "octopus",
    ],
    "white-meat": [
        "poultry",
    ],
    "red-meat": [
        "game bird",
        "beef",
        "pork",
        "sausage",
        "game",
        "lamb",
        "bacon",
        "liver",
    ],
    "sweets": [
        "dessert",
    ],
    "other": [
        "soup",
        "spicy food",
        "niche",
        "ham",
        "sandwich",
        "pizza",
        "pasta",
        "casserole",
        "vegetarian",
        "salad",
    ],
}
